@@ -68,6 +68,7 @@ typedef struct
         bool saveRepeatShuffleSettings;                 // Save repeat and shuffle settings between sessions. Default on.
         int repeatState;                                // 0=disabled,1=repeat track ,2=repeat list
         bool shuffleEnabled;
+        bool trackTitleAsWindowTitle;                   // Set the window title to the title of the currently playing track
 } UISettings;
 
 typedef struct
@@ -140,7 +141,7 @@ typedef struct
         char seekBackward[6];
         char seekForward[6];
         char savePlaylist[6];
-        char addToMainPlaylist[6];
+        char addToFavoritesPlaylist[6];
         char updateLibrary[6];
         char quit[6];
         char altQuit[6];
@@ -216,6 +217,7 @@ typedef struct
         char saveRepeatShuffleSettings[2];
         char repeatState[2];
         char shuffleEnabled[2];
+        char trackTitleAsWindowTitle[2];
 } AppSettings;
 
 #endif
