@@ -15,7 +15,7 @@ enum EventType
         EVENT_TOGGLEREPEAT,
         EVENT_TOGGLEVISUALIZER,
         EVENT_TOGGLEASCII,
-        EVENT_ADDTOMAINPLAYLIST,
+        EVENT_ADDTOFAVORITESPLAYLIST,
         EVENT_DELETEFROMMAINPLAYLIST,
         EVENT_EXPORTPLAYLIST,
         EVENT_UPDATELIBRARY,
