@@ -154,13 +154,13 @@ enum EventType getMouseLastRowEvent(int mouseXOnLastRow)
 
 bool mouseInputHandled(char *seq, int i, struct Event *event)
 {
-        int term_w, term_h;
-        getTermSize(&term_w, &term_h);
-        (void)term_w;
+        char tmpSeq[MAX_SEQ_LEN];
+
+        c_strcpy(tmpSeq, seq, MAX_SEQ_LEN - 1);
 
         int mouseButton, mouseX, mouseY;
         mouseButton = mouseX = mouseY = 0;
-        char *mouseTmp = strtok(seq + 3, ";");
+        char *mouseTmp = strtok(tmpSeq + 3, ";");
 
         if (mouseTmp != NULL)
                 mouseButton = getNumber(mouseTmp);
@@ -171,8 +171,6 @@ bool mouseInputHandled(char *seq, int i, struct Event *event)
         if (mouseTmp != NULL)
                 mouseY = getNumber(mouseTmp);
 
-        int indent = getIndent();
-
         if (progressBarLength > 0)
         {
                 draggedProgressBarCol = mouseX - progressBarCol;
@@ -183,7 +181,7 @@ bool mouseInputHandled(char *seq, int i, struct Event *event)
         }
 
         // Clicked on last row
-        if (mouseY == lastRowRow && indent > 0 &&
+        if (mouseY == lastRowRow && lastRowCol > 0 &&
             mouseX - lastRowCol > 0 && mouseX - lastRowCol < (int)strlen(LAST_ROW) &&
             mouseButton != MOUSE_DRAG)
         {
@@ -433,6 +431,7 @@ void setEndOfListReached(AppState *state)
         audioData.currentFileIndex = 0;
         audioData.restart = true;
         loadingdata.loadA = true;
+        waitingForNext = true;
 
         pthread_mutex_lock(&dataSourceMutex);
         cleanupPlaybackDevice();
@@ -540,6 +539,8 @@ void refreshPlayer(UIState *uis)
 
 void resetListAfterDequeuingPlayingSong(AppState *state)
 {
+        startFromTop = true;
+
         if (lastPlayedId < 0)
                 return;
 
@@ -572,7 +573,6 @@ void resetListAfterDequeuingPlayingSong(AppState *state)
                 audioData.currentFileIndex = 0;
                 audioData.restart = true;
                 waitingForNext = true;
-                startFromTop = true;
                 loadingdata.loadA = true;
                 usingSongDataA = false;
 
@@ -662,7 +662,36 @@ void handleGoToSong(AppState *state)
 
         if (state->currentView == LIBRARY_VIEW)
         {
-                enqueue(state, getCurrentLibEntry());
+                FileSystemEntry *entry = getCurrentLibEntry();
+
+                if (entry == NULL)
+                        return;
+
+                // Enqueue playlist
+                if (pathEndsWith(entry->fullPath, "m3u") || pathEndsWith(entry->fullPath, "m3u8"))
+                {
+                        FileSystemEntry *firstEnqueuedEntry = NULL;
+                        Node *prevTail = playlist.tail;
+
+                        readM3UFile(entry->fullPath, &playlist, library);
+
+                        if (prevTail != NULL && prevTail->next != NULL)
+                        {
+                                firstEnqueuedEntry = findCorrespondingEntry(library, prevTail->next->song.filePath);
+                        }
+                        else if (playlist.head != NULL)
+                        {
+                                firstEnqueuedEntry = findCorrespondingEntry(library, playlist.head->song.filePath);
+                        }
+
+                        autostartIfStopped(firstEnqueuedEntry);
+
+                        markListAsEnqueued(library, &playlist);
+
+                        deepCopyPlayListOntoList(&playlist, unshuffledPlaylist);
+                }
+                else
+                        enqueue(state, entry); // Enqueue song
         }
         else if (state->currentView == SEARCH_VIEW)
         {
@@ -1311,10 +1340,10 @@ void cleanupOnExit()
         deleteCache(appState.tmpCache);
         freeMainDirectoryTree(&appState);
         deletePlaylist(&playlist);
-        deletePlaylist(originalPlaylist);
+        deletePlaylist(unshuffledPlaylist);
         deletePlaylist(favoritesPlaylist);
         free(favoritesPlaylist);
-        free(originalPlaylist);
+        free(unshuffledPlaylist);
         setDefaultTextColor();
         pthread_mutex_destroy(&(loadingdata.mutex));
         pthread_mutex_destroy(&(playlist.mutex));
@@ -1360,11 +1389,11 @@ void cleanupOnExit()
 
 void run(AppState *state, bool startPlaying)
 {
-        if (originalPlaylist == NULL)
+        if (unshuffledPlaylist == NULL)
         {
 
-                originalPlaylist = malloc(sizeof(PlayList));
-                *originalPlaylist = deepCopyPlayList(&playlist);
+                unshuffledPlaylist = malloc(sizeof(PlayList));
+                *unshuffledPlaylist = deepCopyPlayList(&playlist);
         }
 
         if (state->uiSettings.saveRepeatShuffleSettings)
@@ -1842,9 +1871,8 @@ void initState(AppState *state)
         state->tmpCache = NULL;
 }
 
-void initializeStateAndSettings(AppState *appState, AppSettings *settings)
+void initSettings(AppState *appState, AppSettings *settings)
 {
-        initState(appState);
         getConfig(settings, &(appState->uiSettings));
         userData.replayGainCheckFirst = appState->uiSettings.replayGainCheckFirst;
         mapSettingsToKeys(settings, &(appState->uiSettings), keyMappings);
@@ -1854,6 +1882,8 @@ void initializeStateAndSettings(AppState *appState, AppSettings *settings)
 
 int main(int argc, char *argv[])
 {
+        initState(&appState);
+
         UISettings *ui = &(appState.uiSettings);
 
         exitIfAlreadyRunning();
@@ -1869,7 +1899,7 @@ int main(int argc, char *argv[])
                 exit(0);
         }
 
-        initializeStateAndSettings(&appState, &settings);
+        initSettings(&appState, &settings);
 
         if (argc == 3 && (strcmp(argv[1], "path") == 0))
         {
