@@ -244,12 +244,16 @@ int printLogo(SongData *songData, UISettings *ui)
                         title[MAXPATHLEN - 1] = '\0';
                 }
 
-                shortenString(title, term_w - indent - indent - logoWidth - 4);
+                int maxWidth = term_w - indent - indent - logoWidth - 4;
+                char output[MAXPATHLEN + 1];
+                output[0] = '\0';
+
+                processName(title, output, maxWidth, false, false);
 
                 if (ui->useConfigColors)
                         setTextColor(ui->titleColor);
 
-                printf(" %s\n\n", title);
+                printf(" %s\n\n", output);
                 height += 2;
         }
         else
@@ -345,7 +349,7 @@ void printTitleWithDelay(int row, int col, const char *text, int delay, int maxW
 
         printf("\033[%d;%dH", row, col);
         printf("\033[K");
-        printf("%.*s", maxWidth, text);
+        printf("%s", text);
         printf("\n");
         fflush(stdout);
 }
@@ -401,12 +405,10 @@ void printBasicMetadata(int row, int col, int maxWidth, TagSettings const *metad
         if (strnlen(metadata->title, METADATA_MAX_LENGTH) > 0)
         {
                 // Clean up title before printing
-                int titleLength = strnlen(metadata->title, maxWidth);
-                char prettyTitle[titleLength + 1];
+                char prettyTitle[MAXPATHLEN + 1];
+                prettyTitle[0] = '\0';
 
-                c_strcpy(prettyTitle, metadata->title, titleLength + 1);
-
-                trim(prettyTitle, titleLength);
+                processName(metadata->title, prettyTitle, maxWidth, false, false);
 
                 printTitleWithDelay(row, col + 1, prettyTitle, ui->titleDelay, maxWidth);
         }
@@ -911,7 +913,7 @@ void flipNextPage(void)
         else if (appState.currentView == PLAYLIST_VIEW)
         {
                 chosenRow += maxListSize - 1;
-                chosenRow = (chosenRow >= originalPlaylist->count) ? originalPlaylist->count - 1 : chosenRow;
+                chosenRow = (chosenRow >= unshuffledPlaylist->count) ? unshuffledPlaylist->count - 1 : chosenRow;
                 refresh = true;
         }
         else if (appState.currentView == SEARCH_VIEW)
@@ -951,7 +953,7 @@ void scrollNext(void)
         if (appState.currentView == PLAYLIST_VIEW)
         {
                 chosenRow++;
-                chosenRow = (chosenRow >= originalPlaylist->count) ? originalPlaylist->count - 1 : chosenRow;
+                chosenRow = (chosenRow >= unshuffledPlaylist->count) ? unshuffledPlaylist->count - 1 : chosenRow;
                 refresh = true;
         }
         else if (appState.currentView == LIBRARY_VIEW)
@@ -991,9 +993,9 @@ void scrollPrev(void)
 
 int getRowWithinBounds(int row)
 {
-        if (row >= originalPlaylist->count)
+        if (row >= unshuffledPlaylist->count)
         {
-                row = originalPlaylist->count - 1;
+                row = unshuffledPlaylist->count - 1;
         }
 
         if (row < 0)
@@ -1209,7 +1211,7 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                 maxNameWidth = 0;
 
         char dirName[maxNameWidth + 1];
-        char filename[maxNameWidth + 1];
+        char filename[MAXPATHLEN + 1];
         bool foundChosen = false;
         int foundCurrent = 0;
         int extraIndent = 0;
@@ -1394,7 +1396,7 @@ int displayTree(FileSystemEntry *root, int depth, int maxListSize, int maxNameWi
                                 }
                                 else
                                 {
-                                        processName(root->name, filename, maxNameWidth - extraIndent);
+                                        processName(root->name, filename, maxNameWidth - extraIndent, true, true);
                                 }
 
                                 printf("└─ ");
@@ -1824,7 +1826,7 @@ int printPlayer(SongData *songdata, double elapsedSeconds, AppSettings *settings
         }
         else if (state->currentView == PLAYLIST_VIEW && refresh)
         {
-                showPlaylist(songdata, originalPlaylist, &chosenRow, &(uis->chosenNodeId), state, settings);
+                showPlaylist(songdata, unshuffledPlaylist, &chosenRow, &(uis->chosenNodeId), state, settings);
                 state->uiState.resetPlaylistDisplay = false;
                 fflush(stdout);
         }
