@@ -17,13 +17,22 @@ ma_context context;
 
 bool isContextInitialized = false;
 
+bool tryAgain = false;
+
 UserData userData;
 
 ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
 {
         char *filePath = NULL;
 
-        filePath = (pAudioData->currentFileIndex == 0) ? pUserData->songdataA->filePath : pUserData->songdataB->filePath;
+        SongData *songData = (pAudioData->currentFileIndex == 0) ? pUserData->songdataA : pUserData->songdataB;
+
+        if (songData == NULL)
+        {
+                return MA_ERROR;
+        }
+
+        filePath = songData->filePath;
 
         pAudioData->pUserData = pUserData;
         pAudioData->currentPCMFrame = 0;
@@ -38,7 +47,7 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                 pAudioData->format = first->outputFormat;
                 pAudioData->channels = first->outputChannels;
                 pAudioData->sampleRate = first->outputSampleRate;
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
         }
         else if (pathEndsWith(filePath, "opus"))
         {
@@ -47,8 +56,8 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 ma_libopus *first = getFirstOpusDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                ma_libopus_ds_get_data_format(first, &pAudioData->format, &pAudioData->channels, &pAudioData->sampleRate, channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                ma_libopus_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -60,8 +69,21 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
                         return -1;
                 ma_libvorbis *first = getFirstVorbisDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                ma_libvorbis_ds_get_data_format(first, &pAudioData->format, &pAudioData->channels, &pAudioData->sampleRate, channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                ma_libvorbis_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
+                ma_data_source_base *base = (ma_data_source_base *)first;
+                base->pCurrent = first;
+                first->pReadSeekTellUserData = pAudioData;
+        }
+        else if (pathEndsWith(filePath, "webm"))
+        {
+                int result = prepareNextWebmDecoder(songData);
+                if (result < 0)
+                        return -1;
+                ma_webm *first = getFirstWebmDecoder();
+                ma_channel channelMap[MA_MAX_CHANNELS];
+                ma_webm_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -70,13 +92,13 @@ ma_result initFirstDatasource(AudioData *pAudioData, UserData *pUserData)
         {
 #ifdef USE_FAAD
 
-                int result = prepareNextM4aDecoder(filePath);
+                int result = prepareNextM4aDecoder(songData);
                 if (result < 0)
                         return -1;
                 m4a_decoder *first = getFirstM4aDecoder();
                 ma_channel channelMap[MA_MAX_CHANNELS];
-                m4a_decoder_ds_get_data_format(first, &pAudioData->format, &pAudioData->channels, &pAudioData->sampleRate, channelMap, MA_MAX_CHANNELS);
-                ma_data_source_get_length_in_pcm_frames(first, &pAudioData->totalFrames);
+                m4a_decoder_ds_get_data_format(first, &(pAudioData->format), &(pAudioData->channels), &(pAudioData->sampleRate), channelMap, MA_MAX_CHANNELS);
+                ma_data_source_get_length_in_pcm_frames(first, &(pAudioData->totalFrames));
                 ma_data_source_base *base = (ma_data_source_base *)first;
                 base->pCurrent = first;
                 first->pReadSeekTellUserData = pAudioData;
@@ -118,7 +140,7 @@ int createDevice(UserData *userData, ma_device *device, ma_context *context, ma_
 
         result = ma_device_start(device);
         if (result != MA_SUCCESS)
-                return -1;        
+                return -1;
 
         appState.uiState.doNotifyMPRISPlaying = true;
 
@@ -152,7 +174,7 @@ int vorbis_createAudioDevice(UserData *userData, ma_device *device, ma_context *
         result = ma_device_init(context, &deviceConfig, device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize miniaudio device.\n");
+                setErrorMessage("Failed to initialize miniaudio device.");
                 return -1;
         }
 
@@ -161,10 +183,10 @@ int vorbis_createAudioDevice(UserData *userData, ma_device *device, ma_context *
         result = ma_device_start(device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to start miniaudio device.\n");
+                setErrorMessage("Failed to start miniaudio device.");
                 return -1;
         }
-        
+
         appState.uiState.doNotifyMPRISPlaying = true;
 
         return 0;
@@ -178,7 +200,7 @@ int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *con
         result = initFirstDatasource(&audioData, userData);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize m4a file.\n");
+                setErrorMessage("M4a type not supported.");
                 return -1;
         }
         m4a_decoder *decoder = getFirstM4aDecoder();
@@ -193,7 +215,7 @@ int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *con
         result = ma_device_init(context, &deviceConfig, device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize miniaudio device.\n");
+                setErrorMessage("Failed to initialize miniaudio device.");
                 return -1;
         }
 
@@ -203,10 +225,10 @@ int m4a_createAudioDevice(UserData *userData, ma_device *device, ma_context *con
 
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to start miniaudio device.\n");
+                setErrorMessage("Failed to start miniaudio device.");
                 return -1;
         }
-       
+
         appState.uiState.doNotifyMPRISPlaying = true;
 
         return 0;
@@ -236,7 +258,7 @@ int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
         result = ma_device_init(context, &deviceConfig, device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to initialize miniaudio device.\n");
+                setErrorMessage("Failed to initialize miniaudio device.");
                 return -1;
         }
 
@@ -245,10 +267,50 @@ int opus_createAudioDevice(UserData *userData, ma_device *device, ma_context *co
         result = ma_device_start(device);
         if (result != MA_SUCCESS)
         {
-                printf("\n\nFailed to start miniaudio device.\n");
+                setErrorMessage("Failed to start miniaudio device.");
                 return -1;
         }
-        
+
+        appState.uiState.doNotifyMPRISPlaying = true;
+
+        return 0;
+}
+
+int webm_createAudioDevice(UserData *userData, ma_device *device, ma_context *context)
+{
+        ma_result result;
+
+        result = initFirstDatasource(&audioData, userData);
+        if (result != MA_SUCCESS)
+        {
+                printf("\n\nFailed to initialize webm file.\n");
+                return -1;
+        }
+        ma_webm *webm = getFirstWebmDecoder();
+        ma_device_config deviceConfig = ma_device_config_init(ma_device_type_playback);
+
+        deviceConfig.playback.format = audioData.format;
+        deviceConfig.playback.channels = audioData.channels;
+        deviceConfig.sampleRate = audioData.sampleRate;
+        deviceConfig.dataCallback = webm_on_audio_frames;
+        deviceConfig.pUserData = webm;
+
+        result = ma_device_init(context, &deviceConfig, device);
+        if (result != MA_SUCCESS)
+        {
+                setErrorMessage("Failed to initialize miniaudio device.");
+                return -1;
+        }
+
+        setVolume(getCurrentVolume());
+
+        result = ma_device_start(device);
+        if (result != MA_SUCCESS)
+        {
+                setErrorMessage("Failed to start miniaudio device.");
+                return -1;
+        }
+
         appState.uiState.doNotifyMPRISPlaying = true;
 
         return 0;
@@ -265,7 +327,29 @@ bool validFilePath(char *filePath)
         return true;
 }
 
-bool tryAgain = false;
+long long getFileSize(const char *filename)
+{
+        struct stat st;
+        if (stat(filename, &st) == 0)
+        {
+                return (long long)st.st_size;
+        }
+        else
+        {
+                return -1;
+        }
+}
+
+int calcAvgBitRate(double duration, const char *filePath)
+{
+        long long fileSize = getFileSize(filePath); // in bytes
+        int avgBitRate = 0;
+
+        if (duration > 0.0)
+                avgBitRate = (int)((fileSize * 8.0) / duration / 1000.0); // use 1000 for kbps
+
+        return avgBitRate;
+}
 
 int switchAudioImplementation(void)
 {
@@ -324,6 +408,18 @@ int switchAudioImplementation(void)
                                                        channels == decoder->outputChannels &&
                                                        format == decoder->outputFormat));
 
+                if (pathEndsWith(filePath, ".mp3") && userData.currentSongData)
+                {
+                        int avgBitRate = calcAvgBitRate(userData.currentSongData->duration, filePath);
+
+                        if (avgBitRate > 320)
+                                avgBitRate = 320;
+
+                        userData.currentSongData->avgBitRate = audioData.avgBitRate = avgBitRate;
+                }
+                else
+                        audioData.avgBitRate = 0;
+
                 if (isRepeatEnabled() || !(sameFormat && currentImplementation == BUILTIN))
                 {
                         setImplSwitchReached();
@@ -334,14 +430,11 @@ int switchAudioImplementation(void)
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-#ifdef USE_FAAD                        
-                        resetM4aDecoders();
-#endif                        
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+
                         int result = builtin_createAudioDevice(&userData, getDevice(), &context, &builtin_file_data_source_vtable);
 
                         if (result < 0)
@@ -391,14 +484,12 @@ int switchAudioImplementation(void)
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-#ifdef USE_FAAD                        
-                        resetM4aDecoders();
-#endif                        
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+                        audioData.avgBitRate = 0;
+
                         int result = opus_createAudioDevice(&userData, getDevice(), &context);
 
                         if (result < 0)
@@ -438,6 +529,11 @@ int switchAudioImplementation(void)
                                                        channels == nChannels &&
                                                        sampleRate == nSampleRate));
 
+                if (userData.currentSongData)
+                        userData.currentSongData->avgBitRate = audioData.avgBitRate = calcAvgBitRate(userData.currentSongData->duration, filePath);
+                else
+                        audioData.avgBitRate = 0;
+
                 if (isRepeatEnabled() || !(sameFormat && currentImplementation == VORBIS))
                 {
                         setImplSwitchReached();
@@ -448,14 +544,11 @@ int switchAudioImplementation(void)
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-#ifdef USE_FAD                        
-                        resetM4aDecoders();
-#endif                        
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+
                         int result = vorbis_createAudioDevice(&userData, getDevice(), &context);
 
                         if (result < 0)
@@ -473,6 +566,65 @@ int switchAudioImplementation(void)
                         setImplSwitchNotReached();
                 }
         }
+        else if (pathEndsWith(filePath, "webm"))
+        {
+                ma_uint32 sampleRate;
+                ma_uint32 channels;
+                ma_format format;
+                ma_channel channelMap[MA_MAX_CHANNELS];
+
+                ma_uint32 nSampleRate;
+                ma_uint32 nChannels;
+                ma_format nFormat;
+                ma_channel nChannelMap[MA_MAX_CHANNELS];
+                ma_webm *decoder = getCurrentWebmDecoder();
+
+                getWebmFileInfo(filePath, &format, &channels, &sampleRate, channelMap);
+
+                if (decoder != NULL)
+                        ma_webm_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
+
+                bool sameFormat = false;
+
+                // FIXME: Gapless/chaining of decoders disabled for now
+                //bool sameFormat = (decoder != NULL && (format == decoder->format &&
+                //                                       channels == nChannels &&
+                //                                       sampleRate == nSampleRate));
+
+                audioData.avgBitRate = 0;
+
+                if (isRepeatEnabled() || !(sameFormat && currentImplementation == WEBM))
+                {
+                        setImplSwitchReached();
+
+                        pthread_mutex_lock(&dataSourceMutex);
+
+                        setCurrentImplementationType(WEBM);
+
+                        cleanupPlaybackDevice();
+
+                        resetAllDecoders();
+                        resetAudioBuffer();
+
+                        audioData.sampleRate = sampleRate;
+
+                        int result = webm_createAudioDevice(&userData, getDevice(), &context);
+
+                        if (result < 0)
+                        {
+                                setCurrentImplementationType(NONE);
+                                setImplSwitchNotReached();
+                                setEOFReached();
+                                free(filePath);
+                                pthread_mutex_unlock(&dataSourceMutex);
+                                return -1;
+                        }
+
+                        pthread_mutex_unlock(&dataSourceMutex);
+
+                        setImplSwitchNotReached();
+                }
+        }
         else if (pathEndsWith(filePath, "m4a") || pathEndsWith(filePath, "aac"))
         {
 #ifdef USE_FAAD
@@ -484,17 +636,24 @@ int switchAudioImplementation(void)
                 ma_uint32 nSampleRate;
                 ma_uint32 nChannels;
                 ma_format nFormat;
+                int avgBitRate;
                 ma_channel nChannelMap[MA_MAX_CHANNELS];
                 m4a_decoder *decoder = getCurrentM4aDecoder();
+                k_m4adec_filetype fileType = k_unknown;
 
-                getM4aFileInfo(filePath, &format, &channels, &sampleRate, channelMap);
+                getM4aFileInfo(filePath, &format, &channels, &sampleRate, channelMap, &avgBitRate, &fileType);
 
                 if (decoder != NULL)
                         m4a_decoder_ds_get_data_format(decoder, &nFormat, &nChannels, &nSampleRate, nChannelMap, MA_MAX_CHANNELS);
 
                 bool sameFormat = (decoder != NULL && (format == decoder->format &&
                                                        channels == nChannels &&
-                                                       sampleRate == nSampleRate));
+                                                       sampleRate == nSampleRate &&
+                                                       decoder->fileType == fileType &&
+                                                       decoder->fileType != k_rawAAC));
+
+                if (userData.currentSongData)
+                        userData.currentSongData->avgBitRate = audioData.avgBitRate = avgBitRate;
 
                 if (isRepeatEnabled() || !(sameFormat && currentImplementation == M4A))
                 {
@@ -506,12 +665,11 @@ int switchAudioImplementation(void)
 
                         cleanupPlaybackDevice();
 
-                        resetDecoders();
-                        resetVorbisDecoders();
-                        resetM4aDecoders();
-                        resetOpusDecoders();
+                        resetAllDecoders();
                         resetAudioBuffer();
 
+                        audioData.sampleRate = sampleRate;
+
                         int result = m4a_createAudioDevice(&userData, getDevice(), &context);
 
                         if (result < 0)
@@ -576,3 +734,30 @@ int createAudioDevice()
 
         return 0;
 }
+
+void resumePlayback(void)
+{
+        // If this was unpaused with no song loaded
+        if (audioData.restart)
+        {
+                audioData.endOfListReached = false;
+        }
+
+        if (!ma_device_is_started(&device))
+        {
+                if (ma_device_start(&device) != MA_SUCCESS)
+                {
+                        createAudioDevice();
+                        ma_device_start(&device);
+                }
+        }
+
+        paused = false;
+
+        stopped = false;
+
+        if (appState.currentView != TRACK_VIEW)
+        {
+                refresh = true;
+        }
+}
