@@ -16,6 +16,7 @@
 #include "songloader.h"
 #include "soundbuiltin.h"
 #include "soundcommon.h"
+#include "common.h"
 
 #ifndef USERDATA_STRUCT
 #define USERDATA_STRUCT
@@ -40,11 +41,12 @@ typedef struct
         ma_uint32 channels;
         ma_uint32 sampleRate;
         ma_uint32 currentPCMFrame;
+        ma_uint32 avgBitRate;
         bool switchFiles;
         int currentFileIndex;
         ma_uint64 totalFrames;
         bool endOfListReached;
-        bool restart;     
+        bool restart;
 } AudioData;
 #endif
 
@@ -56,6 +58,8 @@ int createAudioDevice();
 
 int switchAudioImplementation(void);
 
+void resumePlayback(void);
+
 void cleanupAudioContext(void);
 
 #endif
