@@ -62,4 +62,10 @@ void enableTerminalMouseButtons(void);
 
 void disableTerminalMouseButtons(void);
 
+void setTerminalWindowTitle(char *title);
+
+void saveTerminalWindowTitle();
+
+void restoreTerminalWindowTitle();
+
 #endif
